"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.request = void 0;
const type_is_1 = require("type-is");
const accepts = require("accepts");
const qs = require("querystring");
const only = require("only");
const util_1 = require("./util");
const BODY = Symbol.for('ctx#body');
exports.request = {
    _accept: null,
    req: null,
    get host() {
        return this.get('host');
    },
    /**
     * Parse the "Host" header field hostname
     *
     * @return {String} hostname
     * @api public
     */
    get hostname() {
        const host = this.host;
        if (!host)
            return '';
        if ('[' === host[0])
            return ''; // IPv6 not support
        return host.split(':', 1)[0];
    },
    accepts(...args) {
        return this.accept.types(...args);
    },
    get accept() {
        return this._accept || (this._accept = accepts(this));
    },
    /**
     * Return accepted encodings or best fit based on `encodings`.
     *
     * Given `Accept-Encoding: gzip, deflate`
     * an array sorted by quality is returned:
     *
     *     ['gzip', 'deflate']
     *
     * @param {String|Array} encoding(s)...
     * @return {String|Array}
     * @api public
     */
    acceptsEncodings(...args) {
        return this.accept.encodings(...args);
    },
    /**
     * Return accepted charsets or best fit based on `charsets`.
     *
     * Given `Accept-Charset: utf-8, iso-8859-1;q=0.2, utf-7;q=0.5`
     * an array sorted by quality is returned:
     *
     *     ['utf-8', 'utf-7', 'iso-8859-1']
     *
     * @param {String|Array} charset(s)...
     * @return {String|Array}
     * @api public
     */
    acceptsCharsets(...args) {
        return this.accept.charsets(...args);
    },
    /**
     * Return accepted languages or best fit based on `langs`.
     *
     * Given `Accept-Language: en;q=0.8, es, pt`
     * an array sorted by quality is returned:
     *
     *     ['es', 'pt', 'en']
     *
     * @param {String|Array} lang(s)...
     * @return {Array|String}
     * @api public
     */
    acceptsLanguages(...args) {
        return this.accept.languages(...args);
    },
    /**
     * faas origin context object
     */
    get originEvent() {
        var _a, _b;
        return ((_b = (_a = this.req).getOriginEvent) === null || _b === void 0 ? void 0 : _b.call(_a)) || {};
    },
    get originContext() {
        var _a, _b;
        return ((_b = (_a = this.req).getOriginContext) === null || _b === void 0 ? void 0 : _b.call(_a)) || {};
    },
    get ip() {
        var _a;
        return ((_a = this.req) === null || _a === void 0 ? void 0 : _a.clientIP) || this.req.ip;
    },
    get url() {
        return this.req.url;
    },
    get path() {
        return this.req.path;
    },
    get method() {
        return this.req.method;
    },
    get headers() {
        return this.req.headers;
    },
    get header() {
        return this.req.headers;
    },
    get query() {
        var _a;
        return ((_a = this.req) === null || _a === void 0 ? void 0 : _a.queries) || this.req.query;
    },
    get body() {
        if (this[BODY]) {
            return this[BODY];
        }
        let body = this.req.body;
        if (util_1.isPlainObject(body)) {
            // body has been parsed in express environment
            this.req.bodyParsed = true;
        }
        if (this.req.bodyParsed) {
            // api 网关会被 parse，这里就直接返回了
            this[BODY] = this.req.body;
            return this[BODY];
        }
        // fc http parser 会走到这里，因为 req.body 是 buffer，且不会被处理
        if (Buffer.isBuffer(body)) {
            body = Buffer.from(body).toString();
        }
        switch (type_is_1.is(this.get('content-type'), ['urlencoded', 'json'])) {
            case 'json':
                try {
                    this[BODY] = JSON.parse(body);
                }
                catch (_a) {
                    throw new Error('invalid json received');
                }
                break;
            case 'urlencoded':
                try {
                    this[BODY] = qs.parse(body);
                }
                catch (_b) {
                    throw new Error('invalid urlencoded received');
                }
                break;
            default:
                this[BODY] = body;
        }
        return this[BODY];
    },
    get params() {
        return this.req.pathParameters || {};
    },
    is(type, ...types) {
        return type_is_1.is(type, ...types);
    },
    /**
     * Return request header.
     *
     * The `Referrer` header field is special-cased,
     * both `Referrer` and `Referer` are interchangeable.
     *
     * Examples:
     *
     *     this.get('Content-Type');
     *     // => "text/plain"
     *
     *     this.get('content-type');
     *     // => "text/plain"
     *
     *     this.get('Something');
     *     // => ''
     *
     * @param {String} field
     * @return {String}
     * @api public
     */
    get(field) {
        switch ((field = field.toLowerCase())) {
            case 'referer':
            case 'referrer':
                return this.headers.referrer || this.headers.referer || '';
            default:
                return this.headers[field] || '';
        }
    },
    /**
     * Inspect implementation.
     *
     * @return {Object}
     * @api public
     */
    inspect() {
        if (!this.req)
            return;
        return this.toJSON();
    },
    /**
     * Return JSON representation.
     *
     * @return {Object}
     * @api public
     */
    toJSON() {
        return only(this, ['method', 'url', 'header']);
    },
    get protocol() {
        // TODO 现在函数没有透出协议
        const proto = this.get('X-Forwarded-Proto');
        return proto ? proto.split(/\s*,\s*/, 1)[0] : 'http';
    },
    /**
     * Short-hand for:
     *
     *    this.protocol == 'https'
     *
     * @return {Boolean}
     * @api public
     */
    get secure() {
        // TODO 现在函数没有透出协议
        return false;
    },
};
//# sourceMappingURL=request.js.map